
#import <Foundation/Foundation.h>

#define LOOPS 10000000

// code copied from CoreFoundation: CFUtilities.c/1.14

#define ELF_STEP(B) T1 = (H << 4) + B; T2 = T1 & 0xF0000000; if (T2) T1 ^= (T2 >> 24); T1 &= (~T2); H = T1
;

unsigned int  elf_hash( unsigned char *bytes, unsigned int length)
{
    /* The ELF hash algorithm, used in the ELF object file format */
    unsigned int H = 0, T1, T2;
    int rem = length;
    while (3 < rem) {
        ELF_STEP(bytes[length - rem]);
        ELF_STEP(bytes[length - rem + 1]);
        ELF_STEP(bytes[length - rem + 2]);
        ELF_STEP(bytes[length - rem + 3]);
        rem -= 4;
    }
    switch (rem) {
    case 3:  ELF_STEP(bytes[length - 3]);
    case 2:  ELF_STEP(bytes[length - 2]);
    case 1:  ELF_STEP(bytes[length - 1]);
    case 0:  ;
    }
    return H;
}

#undef ELF_STEP



#define MULLE_ELF_STEP( B)	\
do			 	\
{			 	\
   H  = (H << 4) + B;		\
   H ^= (H >> 24) & 0xF0;	\
}				\
while( 0)


unsigned int  mulle_elf_hash( unsigned char *bytes, unsigned int length)
{
    unsigned int H = 0;
    int rem = length;
    while (3 < rem) {
        MULLE_ELF_STEP(bytes[length - rem]);
        MULLE_ELF_STEP(bytes[length - rem + 1]);
        MULLE_ELF_STEP(bytes[length - rem + 2]);
        MULLE_ELF_STEP(bytes[length - rem + 3]);
        rem -= 4;
    }
    switch (rem) {
    case 3:  MULLE_ELF_STEP(bytes[length - 3]);
    case 2:  MULLE_ELF_STEP(bytes[length - 2]);
    case 1:  MULLE_ELF_STEP(bytes[length - 1]);
    case 0:  ;
    }
    return H;
}

#undef MULLE_ELF_STEP


#define MULLE_STEP( B)	   	    \
do			 	    \
{			 	    \
   H  = ((H << 4) | (H >> 28)) + B; \
}				    \
while( 0)


unsigned int  mulle_hash( unsigned char *bytes, unsigned int length)
{
    unsigned int H = 0;
    int rem = length;
    while (3 < rem) {
        MULLE_STEP(bytes[length - rem]);
        MULLE_STEP(bytes[length - rem + 1]);
        MULLE_STEP(bytes[length - rem + 2]);
        MULLE_STEP(bytes[length - rem + 3]);
        rem -= 4;
    }
    switch (rem) {
    case 3:  MULLE_STEP(bytes[length - 3]);
    case 2:  MULLE_STEP(bytes[length - 2]);
    case 1:  MULLE_STEP(bytes[length - 1]);
    case 0:  ;
    }
    return H;
}



int main (int argc, const char *argv[])
{
   NSAutoreleasePool * pool = [[NSAutoreleasePool alloc] init];
   unsigned char	buf[ 32];
   unsigned int         i;
   unsigned int         hash1;
   unsigned int         hash2;
   unsigned int         hash3;

   for( i = 0; i < 32; i++)
      buf[ i] = rand() & 0xFF;

   NSLog( @"Start original");
   for( i = 0; i < LOOPS; i++)
      hash1 = elf_hash( buf, sizeof( buf));
   NSLog( @"End %X", hash1);

   NSLog( @"Start mulle_elf");
   for( i = 0; i < LOOPS; i++)
      hash2 = mulle_elf_hash( buf, sizeof( buf));
   NSLog( @"End %X", hash2);

   NSLog( @"Start mulle");
   for( i = 0; i < LOOPS; i++)
      hash3 = mulle_hash( buf, sizeof( buf));
   NSLog( @"End %X", hash3);
   
   // insert your code here

   [pool release];
   exit(0);       // insure the process exit status is 0
   return 0;      // ...and make main fit the ANSI spec.
}
